# -*- coding: UTF-8 -*-

import sys
import os
import httplib2
import json
from common import logger, config

STATUS_ERROR = 'STATUS_ERROR'  # Ошибка загрузки/выгрузки справочника
STATUS_READY = 'STATUS_READY'  # Справочник успешно загружен/выгружен
STATUS_READY_WITH_ERROR = 'STATUS_READY_WITH_ERROR'  # Справочник загружен с ошибками
STATUS_DURING = 'STATUS_DURING'  # Справочник в данный момент загружается/выгружается

log = logger.Logger().getLogger('Service')

if not config.read(sys.path[0] + "/../config/%s/exchangers.ini" % logger.Logger().location):
    log.error(u"Configuration file 'exchangers.ini' not found. The system 'exchangers' is not running")
    exit(1)
    
cs_host = 'localhost'
cs_port = '8080'
cs_prefix = '/CSrest/rest'
cs_host = config.getString('CS', 'node.host', cs_host)
cs_port = config.getString('CS', 'node.port', cs_port)
cs_prefix = config.getString('CS', 'node.prefix', cs_prefix)

base_url = 'http://' + cs_host + ':' + cs_port + cs_prefix


REST_URL = base_url
SALES_BACKOFFICE_URL = REST_URL + '/sales/backOfficeState'
DISTS_BACKOFFICE_URL = REST_URL + '/dicts/backOfficeState'
SALES_CASHUNLOAD_URL = cs_prefix+'/sales/cashUnloadState'

jsonHeader = {'Content-Type': 'application/json', "Accept": "application/json"}
formats = {'aif': 'ARTIXINTERCHANGE', 'shtrih': 'SHTRIHM', 'smag-ukm': 'SUPERMAG', 'smag-ukm4': 'SUPERMAG4',
           'oracle': 'ORACLE', 'atol': 'ATOL', 'soap': 'ARTIXINTERCHANGE_SOAP', 'crystal': 'CRYSTAL',
           'rest': 'ARTIXINTERCHANGE_REST', 'ns2000': 'NS2000', 'pilot': 'PILOT', 'simple_file': 'SIMPLE-FILE'}
requestTimeout = 3

def convertFileName(fileName):
    if isinstance(fileName, str):
        return os.path.basename(fileName)
    if isinstance(fileName, list):
        result = []
        for f in fileName:
            result.append(os.path.basename(f))
        return str(result)
    return None

def sendDictState(node, messageEn, messageRu, status, fileName=None):
    http = httplib2.Http(timeout=requestTimeout)
    body = {'messageEn': messageEn, 'messageRu': messageRu, 'status': status}
    log = logger.Logger().getLoggerUpload(node.id)
    url = DISTS_BACKOFFICE_URL + '/%s' % node.id
    # отправка статуса только когда задан statusServer
    if hasattr(node.currentConverter, 'statusServer') and node.currentConverter.statusServer:
        url = url.replace('localhost:8080', node.currentConverter.statusServer)

        if node.currentConverter.dict_state_id:
            log.debug('UPDATE dictState')
            method = 'PUT'
            url += '/%s' % node.currentConverter.dict_state_id
        else:
            method = 'POST'
            body['format'] = formats.get(node.currentConverter.name, 'UNKNOWN')
            body['fileName'] = convertFileName(fileName)
        log.debug('send to: %s dictState: %s' % (url,body))
        body = json.dumps(body)
        try:
            response, content = http.request(url, method, headers=jsonHeader, body=body)
            if response.status == 200:
                return json.loads(content)
            else:
                log.warning(u'Ошибка отправки статуса конвертирования справочника на КС. %s' % content)
        except Exception as e:
            log.warning(u'Ошибка отправки статуса конвертирования справочника на КС. %s' % str(e))
    return None

def getDictState(node, fileName):
    http = httplib2.Http(timeout=requestTimeout)
    if logger.Logger().location == 'cs':
        url = DISTS_BACKOFFICE_URL + '/%s?fileName=%s' % (node.id, convertFileName(fileName))
        response, content = http.request(url, 'GET', headers=jsonHeader)
        if response.status == 200:
            dictStates = json.loads(content)
            if dictStates and isinstance(dictStates, list):
                dictStates = [value for value in dictStates if value != None]
                if len(dictStates) > 0:
                    return sorted(dictStates, key=lambda k: k['time'], reverse=True)[0]['id']
    return None

def sendSaleState(node, cashCode, shiftNums, messageEn, messageRu, status, format, resultFiles=None, cashUnloadStateId=None, statusHost=None):
    log = logger.Logger().getLoggerUnload(node.id)
    if logger.Logger().location == 'cs':
        body = {'messageEn': messageEn, 'messageRu': messageRu, 'status': status, 'format': format}
        if resultFiles:
            body['resultFiles'] = resultFiles
        results = []
        body = json.dumps(body)
        log.info('send saleState - body: %s' % body)
        # Строчка ниже получает cashId кассы, зная код кассы и id магазина. Это можно сделать проще и пробросить его сверху,
        # но на текущий момент опасно менять что-то в exchanger за пределами тела метода, т.к. можно нарваться на внезапные проблемы в production
        cashId = filter(lambda x: x.code == cashCode, node.cashes)[0].cashId
        for shiftNum in shiftNums:
            # такой метод должен появиться в КС начиная с версии 3.0.246
            url = SALES_BACKOFFICE_URL + '/append?shopid=%s&cashCode=%s&cashid=%s&shiftNum=%s' % (node.id, cashCode, cashId, shiftNum)
            result = sendState(url, 'POST', body, log)
            if result:
                results.append(result)
        return results
    elif logger.Logger().location == 'cash' and cashUnloadStateId and statusHost:
        return updateCashUnloadStatus(cashUnloadStateId, messageEn, messageRu, status, log, statusHost=statusHost)
    return None
    
    
def updateSaleState(node, saleBackOfficeIds, state, cashUnloadStateId=None, statusHost=None):
    log = logger.Logger().getLoggerUnload(node.id)
    if logger.Logger().location == 'cs':
        results = []
        body = json.dumps(state)
        log.debug('update saleState - ids: %s body: %s' % (saleBackOfficeIds, body))
        for saleBackOfficeId in saleBackOfficeIds:
            url = SALES_BACKOFFICE_URL + '/%s' % saleBackOfficeId
            result = sendState(url, 'PUT', body, log)
            if result:
                results.append(result)
        return results
    elif logger.Logger().location == 'cash' and cashUnloadStateId and statusHost:
        return updateCashUnloadStatus(cashUnloadStateId, state['messageEn'], state['messageRu'], state['status'], log, statusHost=statusHost)
    return None

def updateCashUnloadStatus(cashUnloadStateId, messageEn, messageRu, status, log, statusHost=None):
    if status == STATUS_READY:
        status = STATUS_DURING
    body = json.dumps({'messageEn': messageEn, 'messageRu': messageRu, 'status': status})
    url = 'http://' + statusHost + SALES_CASHUNLOAD_URL + '/%s' % cashUnloadStateId
    return sendState(url, 'PUT', body, log)
    
def sendState(url, method, body, log):
    try:
        http = httplib2.Http(timeout=requestTimeout)
        response, content = http.request(url, method, headers=jsonHeader, body=body)
        if response.status == 200:
            return json.loads(content)
        else:
            log.warning(u'Получен некоректный http-код %s во время отправки статуса на КС: %s' % (response.status, content))
    except Exception as e:
        log.warning(u'Во время отправки статуса на КС произошла ошибка: %s' % str(e))
    return None

def startLoadDictCash(cashCode, log):
    try:
        http = httplib2.Http(timeout=requestTimeout)
        response, content = http.request(REST_URL + '/cashes/%s/dicts/startLoad' % cashCode, 'POST', headers=jsonHeader)
        if response.status == 200:
            log.info(u'Oтправлен сигнал на кассу %s для запуска загрузки справочников.' % cashCode)
        else:
            log.warning(u'Получен некоректный http-код %s во время отправки сигнала на кассу %s для запуска загрузки справочников: %s' % (response.status, cashCode, content))
    except Exception as e:
        log.warning(u'Во время оотправки сигнала на кассу %s для запуска загрузки справочников произошла ошибка: %s' % (cashCode, str(e)))

def rereadQueuesInNES(log, nesHost='localhost:28081'):
    try:
        nesHost = nesHost if nesHost else 'localhost:28081'
        http = httplib2.Http(timeout=requestTimeout)
        response, content = http.request('http://%s/nes/queue_reread' % nesHost, 'PUT', headers=jsonHeader)
        if response.status == 200:
            log.info(u'Oтправлен сигнал в NES, чтобы он перечитал очереди')
        else:
            log.warning(u'Ошибка отправки сигнала в NES, чтобы он перечитал очереди. Он перечитает их позже')
    except:
        log.warning(u'Ошибка отправки сигнала в NES, чтобы он перечитал очереди. Он перечитает их позже')
        
def startLoadNES(log, shopId, nesHost='localhost:28081'):
    try:
        nesHost = nesHost if nesHost else 'localhost:28081'
        http = httplib2.Http(timeout=requestTimeout)
        log.info('send signal to nes on host:%s' % nesHost)
        response, content = http.request('http://%s/nes/queue_load' % nesHost, 'PUT', body='{ "shopId":"%s"}' % shopId, headers=jsonHeader)
        if response.status == 200:
            log.info(u'Oтправлен сигнал в NES, чтобы он приступил к загрузке для магазина:%s' % shopId)
        else:
            log.warning(u'Ошибка отправки сигнала в NES, чтобы он приступил к загрузке для магазина:%s. Он загрузит данные позже' % shopId)
    except:
        log.warning(u'Ошибка отправки сигнала в NES, чтобы он приступил к загрузке для магазина:%s. Он загрузит данные позже' % shopId)
